import os
from pathlib import Path

from . import config as C

def zsh_shell(exe_dir, shell_path):
    rcfile = exe_dir / '.coil.zsh' / '.zshrc'
    rcfile.parent.mkdir(exist_ok=True)
    with open(rcfile, 'w') as f:
        f.write(f'''\
. ~/.zshrc
. {exe_dir / 'activate'}
''')
    os.environ['ZDOTDIR'] = str(rcfile.parent)
    os.execlp(shell_path or 'zsh', '-i')


def bash_shell(exe_dir, shell_path):
    """
    TODO: windows support
    """


def get_shell():
    shell_path = os.environ.get('SHELL')
    if shell_path:
        shell_path = Path(shell_path)
        shell_name = shell_path.name
    else:
        shell_name = C.DEFAULT_SHELL
    return SHELL_COMMANDS.get(shell_name), shell_path


# SHELL_COMMANDS = {
#     'cmd': cmd_shell,
#     'powershell': ps_shell,
#     'ps': ps_shell,
#     'bash': bash_shell,
#     'fish': fish_shell,
#     'zsh': zsh_shell,
#     'xonsh': xonsh_shell,
#     'tcsh': tcsh_shell,
#     'csh': csh_shell,
# }


SHELL_COMMANDS = {
    'zsh': zsh_shell,
    'bash': bash_shell,
}


import json

from .config import ENV_DB


class DB(dict):
    def __init__(self, *a, **b):
        super().__init__(self, *b, **b)
        self.load()

    def load(self):
        self.clear()
        try:
            with open(ENV_DB) as f:
                self.update(json.load(f))
        except FileNotFoundError:
            pass

    def dump(self):
        with open(ENV_DB, 'w') as f:
            json.dump(self, f)



from pathlib import Path
import venv as pyvenv
import random
import shutil
import subprocess

from . import config as C, db, shell


class VenvError(Exception):
    """
    errors managing virtualenvs
    """


def bindir(path):
    d = path / 'bin'
    if d.is_dir():
        return d
    if C.ON_WINDOWS:
        d = path / 'Scripts'
        if d.is_dir():
            return d
    return None


def check(path):
    return bindir(path) is not None


def list_available():
    for l in C.VENV_DIR.glob('*'):
        if check(l):
            yield l.name


def load(path):
    print(f'Loading {path}...')
    bd = bindir(path)
    if bd is None:
        raise VenvError('how did you get here??')
    l, p = shell.get_shell()
    l(bd, p)


def bind(name, path=None):
    if path is None:
        path = Path.cwd()
    C.setup_conf()
    venv_path = C.VENV_DIR / name
    dbi = db.DB()
    path_str = str(path)
    if path_str in dbi:
        raise VenvError(f'{path} is already bound to {path_str}')
    dbi[path_str] = name
    dbi.dump()
    return venv_path


def unbind(path=None):
    env, ppath = for_path_helper(path)
    if env is None:
        raise VenvError(f'{path} is NOT bound to any env')
    dbi = db.DB()
    del dbi[str(ppath)]
    dbi.dump()


def new(name=None):
    if name is None:
        # generate random one
        name = f'auto_{int(random.random()*16777215):x}'
    venv_path = C.VENV_DIR / name
    pyvenv.create(venv_path, with_pip=True)
    return name


def shed(name):
    venv_path = C.VENV_DIR / name
    if not venv_path.exists():
        raise VenvError(f'{name} does not exist')
    shutil.rmtree(venv_path)


def for_path_helper(path=None):
    if path is None:
        path = Path.cwd()
    else:
        path = path.absolute()
    root = Path(path.root)
    while path != root:
        r = db.DB().get(str(path))
        if r is not None:
            return C.VENV_DIR / r, path
        path = path.parent
    return None, path


def for_path(path=None):
    return for_path_helper(path)[0]



import platform
import os
from pathlib import Path


__platform = platform.system()
ON_MACOS = os.name == 'mac' or __platform == 'Darwin'
ON_WINDOWS = os.name == 'nt' or __platform == 'Windows'
DEFAULT_SHELL = 'cmd' if ON_WINDOWS else 'bash'

# TODO: make this configurable
VENV_DIR = Path.home() / '.virtualenvs'
CONF_DIR = Path.home() / '.coil'
ENV_DB = CONF_DIR / 'mapping.json'


def setup_conf():
    CONF_DIR.mkdir(exist_ok=True)



from . import __description__, __version__
import argparse
import sys

from . import venv


def parseargs(args):
    parser = argparse.ArgumentParser(
        add_help=False,
        description=__description__,
        usage='%(prog)s [OPTIONS] ...',
    )
    parser.add_argument('-h', '--help', action='help', help=argparse.SUPPRESS)
    parser.add_argument('-l', '--list', action='store_true', help='list available venvs')
    parser.add_argument('-b', '--bind', type=str, metavar='ENV', help='bind venv to current path')
    parser.add_argument('-u', '--unbind', action='store_true', help='unbind venv from current path')
    parser.add_argument('-c', '--create', type=str, metavar='ENV', help='create new venv and bind it to current path')
    parser.add_argument('-a', '--auto', action='store_true', help='create new venv (random name) and bind it to current path')
    parser.add_argument('-s', '--shed', type=str, nargs='+', help='remove these virtualenvs')
    parser.add_argument('--version', action='version', version=__version__)

    return parser.parse_args(args)


def main_wrapped(args=None):
    args = parseargs(args)
    if args.list:
        for _e in venv.list_available():
            print(_e)
        return 0

    if args.unbind:
        venv.unbind()
        return

    if args.shed:
        for v in args.shed:
            print(f'Shedding {v}')
            venv.shed(v)
        return

    if args.bind:
        env = venv.bind(args.bind)
        if env is None:
            print(f'env {args.bind} does not exist. Use -c create it or -l to see the ones available')
            return 1
    elif args.create:
        venv.new(name=args.create)
        env = venv.bind(args.create)
    elif args.auto:
        env_name = venv.new()
        env = venv.bind(env_name)
    else:
        env = venv.for_path()
        if env is None:
            print('No env configured for current directory. Use "-b ENV" to bind an existing one or "-c ENV" to create and bind to a new one')
            return 1

    venv.load(env)


def main(args=None):
    try:
        main_wrapped(args=args)
    except venv.VenvError as e:
        print(f'ERROR: {e}', file=sys.stderr)
        return 1


if __name__ == '__main__':
    exit(main() or 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
import os
from setuptools import setup, find_packages
import sys

from coil import __program__, __version__, __description__

README = open('README.md').read()

# allow setup.py to be run from any path
os.chdir(os.path.dirname(os.path.abspath(__file__)))

setup(
    name=__program__,
    version=__version__,
    license='MIT',
    description=__description__,
    long_description=README,
    url='https://github.com/fopina/coiled-venv',
    download_url='https://github.com/fopina/coiled-venv/tarball/v%s' % __version__,
    author='Filipe Pina',
    author_email='fopina@skmobi.com',
    packages=find_packages(),
    entry_points={'console_scripts': ['coil=coil.__main__:main']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords=['productivity', 'virtual env', 'cleanup', 'cli'],
)
